//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "GameScenario.h"
#include "MapCreationDialog.h"
#include "GamePreferencesDialog.h"
#include "MapGenerator.h"
#include "armyset.h"
#include "playerlist.h"
#include "citylist.h"
#include "ruinlist.h"
#include "templelist.h"
#include "city.h"
#include "ruin.h"
#include "armysetlist.h"
#include "stacklist.h"
#include "GameMap.h"
#include "player.h"
#include "Configuration.h"
#include "real_player.h"
#include "ai_dummy.h"
#include "ai_fast.h"
#include "counter.h"
#include "army.h"
#include "QuestsManager.h"
// TBD - Port to SDL_net
//#include "Server.h"
//#include "Client.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

GameScenario::GameScenario(GameType gametype, string ip, int port)
	:d_turns(0), d_gameType(SINGLE_PLAYER), d_name("")
{
	debug("GameScenario()")
	
    // create all necessary objects
    // singleton: we don't need the variable now we just create the instance

    QuestsManager::getInstance();

    GameMap::getInstance();
    Armysetlist::getInstance();
    Playerlist::getInstance();
    Citylist::getInstance();
    Templelist::getInstance();
    Ruinlist::getInstance();
    fl_counter = new FL_Counter();

    if (d_gameType == MULTI_PLAYER_HOST)
    {
        /* TBD
        d_server = new Server(port);
        d_server->acceptClients();
        */
    }
    else if (d_gameType == MULTI_PLAYER_CLIENT)
    {
        /* TBD
        d_client = new Client(ip, port);
        */
    }

	initGame();
}

// savegame is a filename with absolute path!

GameScenario::GameScenario(string savegame, bool& broken)
{
    Armysetlist::getInstance()->loadArmyset("default");

    broken = false;
    XML_Helper helper(savegame, ios::in, Configuration::s_zipfiles);

    helper.registerTag("scenario", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("map", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("playerlist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("citylist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("templelist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("ruinlist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("counter", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("questlist", SigC::slot((*this), &GameScenario::load));

    //now parse the document and close the file afterwards
    if (!helper.parse())
    {
        broken = true;
    }

    helper.close();
}

GameScenario::~GameScenario()
{
    // GameMap is a Singleton so we need a function to delete it
    GameMap::deleteInstance();
	Armysetlist::deleteInstance();
    Playerlist::deleteInstance();
    Citylist::deleteInstance();
    Templelist::deleteInstance();
    Ruinlist::deleteInstance();
    QuestsManager::deleteInstance();

    if (fl_counter)
    {
        delete fl_counter;
        fl_counter = 0;
    }
} 

void GameScenario::initGame()
{
    debug("initGame()");

    switch (d_gameType)
    {
        case MULTI_PLAYER_HOST:
            {
                initMultiPlayer();
                break;
            }
        case MULTI_PLAYER_CLIENT:
            {
                initMultiPlayer();
                break;
            }
    }
}

bool GameScenario::isMultiPlayer() const
{
	return (d_gameType == MULTI_PLAYER_HOST) || (d_gameType == MULTI_PLAYER_CLIENT);
}

void GameScenario::initMultiPlayer()
{
	/*TBD
	debug("initMultiPlayer()");
	client->wait_for_map();
	mapdef->random();

	client->wait_for_playerorder();
	load_armyset();

	if (client->is_first())
	{
		assign_players();
		produce_first_army();
	}

	//we just assume, the map from the server is already smoothed, so we
	//don't need to do so
	*/
}

void GameScenario::nextTurn()
{
	if (++d_turns != 0)
	{
        Citylist::getInstance()->nextTurn();
		Stacklist::nextTurn();
	}
}

bool GameScenario::saveGame(string filename)
{
	bool retval = true;

	XML_Helper helper(filename, ios::out, Configuration::s_zipfiles);

	//start writing
    retval &= helper.begin(FREELORDS_SAVEGAME_VERSION);
	retval &= helper.openTag("freelords");
	
	//save the private GameScenario data first
	retval &= helper.openTag("scenario");
	retval &= helper.saveData("name", d_name);
	retval &= helper.saveData("type", d_gameType);
	retval &= helper.saveData("turn", d_turns);
	retval &= helper.closeTag();
	if (!retval)
	{
        std::cerr <<"GameScenario: Something went wrong with saving.\n";
		return false;
	}
	
	//if retval is still true it propably doesn't change throughout the rest
	//now save the single object's data
    retval &= fl_counter->save(&helper);
	retval &= GameMap::getInstance()->save(&helper);
	retval &= Playerlist::getInstance()->save(&helper);
	retval &= Citylist::getInstance()->save(&helper);
	retval &= Templelist::getInstance()->save(&helper);
	retval &= Ruinlist::getInstance()->save(&helper);
	retval &= QuestsManager::getInstance()->save(&helper);

	retval &= helper.closeTag();

	helper.close();

	if (retval)
    {
		return true;
    }
		
    std::cerr <<"GameScenario: Something went wrong with saving.\n";
	return false;
}

bool GameScenario::load(string tag, XML_Helper* helper)
{
    int i;
    
    if (tag == "scenario")
    {
        if (helper->getVersion() != FREELORDS_SAVEGAME_VERSION)
        {
            cerr <<"savefile has wrong version, we want ";
            std::cerr <<FREELORDS_SAVEGAME_VERSION <<",\n";
            cerr <<"savefile offers " <<helper->getVersion() <<".\n";
            return false;
        }
    
        debug("loading scenario")
        helper->getData(d_turns, "turn");
        helper->getData(d_name, "name");
        helper->getData(i, "type");
        d_gameType = static_cast<GameType>(i);

        //TODO: for later when it becomes crucial: deal with loading of
        //something else than simple games
        return true;
    }

    if (tag == "counter")
    {
        debug("loading counter")
        fl_counter = new FL_Counter(helper);
        return true;
    }

    if (tag == "map")
    {
        debug("loading map")
        GameMap::getInstance(helper);
        return true;
    }

    if (tag == "playerlist")
    {
        debug("loading players");
        Playerlist::getInstance(helper);
        return true;
    }

    if (tag == "citylist")
    {
        debug("loading cities")

        Citylist::getInstance(helper);
        return true;
    }

    if (tag == "templelist")
    {
        debug("loading temples")
        Templelist::getInstance(helper);
        return true;
    }

    if (tag == "ruinlist")
    {
        debug("loading ruins")
        Ruinlist::getInstance(helper);
        return true;
    }

    if (tag == "questlist")
    {
        debug("loading quests")
        QuestsManager::getInstance(helper);
        return true;
    }

    return false;
}
