//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "GraphicsCache.h"
#include "armysetlist.h"
#include "armyset_army.h"
#include "playerlist.h"
#include "Configuration.h"


//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)


GraphicsCache* GraphicsCache::s_instance = 0;


GraphicsCache* GraphicsCache::getInstance()
{
    if (!s_instance)
        s_instance = new GraphicsCache();

    return s_instance;
}

void GraphicsCache::deleteInstance()
{
    if (!s_instance)
        return;

    delete s_instance;
    s_instance =0;
}

void GraphicsCache::clearInstance()
{
    if (!s_instance)
        return;

    s_instance->clear();
}

Uint32 GraphicsCache::getCacheSize()
{
    return Configuration::s_cacheSize;
}

GraphicsCache::GraphicsCache()
{
    d_cursize = 0;
}

GraphicsCache::~GraphicsCache()
{
    clear();
}

SDL_Surface* GraphicsCache::getArmyPic(std::string armyset, Uint32 army,
                                        Player* p)
{
    debug("getting army pic" <<armyset <<" " <<army <<" " <<p->getName())

    std::list<GraphicsCacheItem*>::iterator it;
    GraphicsCacheItem* myitem;

    //special situation: ruin keepers don't belong to any player
    if (!p)
        p = Playerlist::getInstance()->getNeutral();
    
    for (it =d_cachelist.begin(); it != d_cachelist.end(); it++)
    {
        if (((*it)->graphset == armyset) && ((*it)->ref == army)
            && ((*it)->player ==p))
        {
            myitem = (*it);
            
            //put the item on the first place (==last touched)
            d_cachelist.erase(it);
            d_cachelist.push_front(myitem);
            
            return myitem->surface;
        }
    }

    //We are still here, so the graphic is not in the cache. addArmyPic calls
    //checkPictures on its own, so we can simply return the surface
    myitem = addArmyPic(armyset, army, p);

    return myitem->surface;
}

void GraphicsCache::checkPictures()
{
    while (d_cursize > Configuration::s_cacheSize)
    {
        //leave at least one element in the list
        if (d_cachelist.size() <= 1)
            return;

        eraseLastItem();
    }
}

GraphicsCacheItem* GraphicsCache::addArmyPic(std::string armyset, Uint32 army,
                                                Player* p)
{
    debug("loading army pic")

    GraphicsCacheItem* myitem = new GraphicsCacheItem();
    myitem->graphset = armyset;
    myitem->ref = army;
    myitem->player = p;

    //Now the most important part: load the army picture
    //First, copy the army picture and change it to the display format
    Armyset_army* basearmy = &(Armysetlist::getInstance()->getArmyset(armyset)->army[army]);
    myitem->surface = SDL_DisplayFormatAlpha(basearmy->getPixmap());

    //Now, if we have a mask ready, copy the mask over the picture. The mask
    //is <= 256 colors, we use the player's palette to colorize the mask. If
    //the armyset has no mask (happens during the transition phase), ignore this
    //step.
    if (basearmy->getMask())
    {
        SDL_Surface* mask = SDL_CreateRGBSurface(SDL_SWSURFACE, 40, 40,
                            basearmy->getMask()->format->BitsPerPixel,0,0,0,0);

        //Set some color palette first!! Else we end up with a black surface.
        SDL_SetColors(mask, basearmy->getMask()->format->palette->colors, 0,
                            basearmy->getMask()->format->palette->ncolors);
                            
        SDL_BlitSurface(basearmy->getMask(), 0, mask, 0);

        //now change tmp's palette to the player's one
        SDL_SetColors(mask, p->getPalette(), 0, 16);

        //set the first pixel as alpha value
        SDL_SetColorKey(mask, SDL_SRCCOLORKEY, 0);

        //blit mask over the army pic
        SDL_BlitSurface(mask, 0, myitem->surface, 0);

        //free the temporary surface
        SDL_FreeSurface(mask);
    }
    
    //now the final preparation steps:
    //a) add the size
    int size = myitem->surface->w * myitem->surface->h;
    d_cursize += myitem->surface->format->BytesPerPixel * size;

    //b) add the entry to the list
    d_cachelist.push_front(myitem);

    //c) check if the size is too large
    checkPictures();

    //we are finished, so return the pic
    return myitem;
}

void GraphicsCache::clear()
{
    while (!d_cachelist.empty())
        eraseLastItem();
}

void GraphicsCache::eraseLastItem()
{
    if (d_cachelist.empty())
        return;

    //As the name suggests, this function erases the last item in the list.
    //Whenever an item is requested, it moves to the first position, so the
    //last item is the oldest and therefore propably most useless in the list.
    GraphicsCacheItem* myitem = *(d_cachelist.rbegin());
    d_cachelist.erase(d_cachelist.end()--);

    //don't forget to subtract the size from the size entry
    int size = myitem->surface->w * myitem->surface->h;
    d_cursize -= myitem->surface->format->BytesPerPixel * size;

    SDL_FreeSurface(myitem->surface);
    delete myitem;
}
