//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/* This is the GraphicsCache class. As the name suggests it is here to cache
 * already prepared graphics.
 * With the new player-specific colors, each time a stack picture is needed, it
 * has to be constructed from a kind of basic stack picture and an overlying
 * mask for the player-specific color regions. This procedure makes the units
 * look cooler, but needs some additional blits. To reduce the overhead from
 * these blits, the graphics are cached in this class. When an adjustable
 * maximum size is reached, the graphics whose use dates back the longest time
 * are deleted.
 *
 * NOTE: For efficiency reasons, the class will not copy the surfaces it has,
 * so DON'T MODIFY THEM unless you know what you do.
 * 
 * About the internals:
 * The pictures are stored in a list of GraphicsCacheItems, which hold the name
 * of the armyset, the army number, the player and the surface itself. When a
 * picture is requested, the class first looks in its cache. If it doesn't find
 * the pic there, it loads it and deletes older pics if the maximum memory
 * consumption has been exceeded.
 */

#ifndef GRAPHICS_CACHE_H
#define GRAPHICS_CACHE_H

#include <paragui.h>
#include <string>
#include <list>

#include "player.h"

//this structure is the base class for storing cached objects. It stores the
//(army-, but can be extended) set, the type of army and the player which the
//surface is designed for (and the surface itself, of course).
typedef struct
{
    std::string graphset;
    Uint32 ref;
    Player* player;
    SDL_Surface* surface;
} GraphicsCacheItem;
    

class GraphicsCache
{
    public:
        //the static functions for accessing the variables
        static GraphicsCache* getInstance();
        static void deleteInstance();
        static void clearInstance();  //don't delete the instance, but clear the
                                      //cache, so clearInstance
        //note that cache size only sets the max size of the pixel information
        static Uint32 getCacheSize();
 
        //here have to come more functions for accessing pictures via the cache
        SDL_Surface* getArmyPic(std::string armyset, Uint32 army, Player* p);
        
    private:
        GraphicsCache();
        ~GraphicsCache();

        //here have to come the private graphic functions
        void checkPictures();   //check if we reached max size and delete if so
        GraphicsCacheItem* addArmyPic(std::string armyset, Uint32 army, Player* p);

        void clear();
        void eraseLastItem();
        
        //the data
        static GraphicsCache* s_instance;

        Uint32 d_cursize;
        std::list<GraphicsCacheItem*> d_cachelist;
};

#endif
