//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef NEXT_TURN_H
#define NEXT_TURN_H

#include <sigc++/object_slot.h>
#include "playerlist.h"

/**
 * \brief The class to pass turns around the players
 *
 * \note    This class takes care of calling the correct players in the correct
 *          order. The problem is, no other class really has the scope of doing
 *          this whole stuff. The playerlist and the W_Edit object aren't the
 *          right candidates (they are busy with other things) and the former
 *          algorithm of each player calling the next one produces a huge
 *          stackload if two ai players fight each other. Plus, you want to do
 *          several actions at the end or the beginning of each player's turn
 *          or each round and therefore want a central place for this code.
 */

class NextTurn: public SigC::Object
{
    public:
        /**
         * \brief the singleton getter
         *
         * we don't offer an explicit deleteInstance function, since this class
         * doesn't hold any variables to be resetted.
         */
        static NextTurn* getInstance();
    
        /**
         * \brief start a new game
         *
         * This function starts with the currently active player, of the first
         * if there is none active. For starting a game.
         */
        void start();

        /**
         * \brief go on to the next player
         * 
         * This function starts the next to the active player's turn. Used when
         * a human player has pushed the next_turn button.
         */
        void endTurn();

        /**
         * \brief signals for announcing events
         */
        SigC::Signal1<void, Player*> splayerStarted;
        SigC::Signal1<void, Player*> splayerFinished;
        SigC::Signal0<void> snextRound;

    private:
        /**
         * \brief constructor
         */
        NextTurn();

        /**
         * \brief The function for all actions which are taken at the end of a
         * player's turn
         */
        void finishTurn();

        /**
         * \brief The function for all actions which are taken at the end of a
         * Game Turn.
         */
        void finishRound();
};

#endif //NEXT_TURN_H
