//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/* This class is intended to make xml handling easier. It will eventually offer
 * save and load functions. You can initialise it either with a filename or
 * a stream (to make dumping data also possible)
 *
 * Saving works like this:
 * Open a new tag with openTag(), save your data with saveData() and close the
 * tag with closeTag(). The output will look like:
 *
 * <map>
 * 	<maptile>
 * 		<d_x> 50 </d_x>
 * 		<d_y> 10 </d_y>
 * 		<d_type> 5 </d_type>
 * 	</maptile>
 * 	...
 * </map>
 *
 * map and maptile are opened tags, the tags x, y and type are saved data.
 * ATTENTION: Always save data before you start opening up subtags!!!
 * i.e. if you want to save some data concerning the tag map, save it _before_
 * you open the first subtag, i.e. maptile
 *
 * Loading:
 * First, you supply xml_helper with callback functions via registerTag.
 * You can also unregister these functions with unregisterTag, but usually this
 * isn't neccessary
 *
 * When you are ready, call parse. The parse will start until it finds a special
 * tag (not starting with d_). It will then parse the next tags, assuming that
 * these are data tags, until it finds the next special tag. When this is
 * reached, it stops and calls the callback for the former special tag. 
 * Sometimes (e.g. in the case of a ruin being created and a stack tag) this
 * callback will set up the callback for the next tag and demand some data.
 * This data has already been stored and will be supplied on a request of
 * getData(). That's all.
 */

#ifndef XML_HELPER_H
#define XML_HELPER_H

#include <string>
#include <iostream>
#include <fstream>
#include <list>
#include <map>
#include <sigc++/sigc++.h>
#include <expat.h>
	
class XML_Helper;

typedef SigC::Slot2<bool, std::string, XML_Helper*> XML_Slot;
//return type (bool), parameters tag(string), this(XML_Helper)

class XML_Helper
{
	public:
        XML_Helper(std::string filename,std::ios::openmode mode,bool zip=false);
			//ios::out for writing (attention, will truncate
			//file!!), in for reading
		        //zip is needed for the zipped and obfuscated savefile  
		XML_Helper(std::istream* input);
			//read xml from input
		XML_Helper(std::ostream* output);
			//write xml to output
		~XML_Helper();

		//save functions
        bool begin(std::string version); //call this at the beginning of saving
		bool openTag(std::string name);
		bool closeTag();
		
		bool saveData(std::string identifier, std::string value);
		bool saveData(std::string identifier, int value);
		bool saveData(std::string identifier, unsigned int value);
		bool saveData(std::string identifier, bool value);
		bool saveData(std::string identifier, double value);

		bool close();
			//Attention: as soon as you call this function, the
			//object is dead with all streams cut. It is just here
			//to force saving of files.

		//load functions
		bool registerTag(std::string tag, XML_Slot callback);
			//registers callbacks for special tags
		bool unregisterTag(std::string tag);

		bool getData(std::string& data, std::string name);
		bool getData(bool& data, std::string name);
		bool getData(int& data, std::string name);
		bool getData(unsigned int& data, std::string name);
		bool getData(double& data, std::string name);
        std::string getVersion() {return d_version;}
		
		bool parse();
		XML_Parser getParser() {return d_parser;}

		//these functions are solely for interaction with the expat
		//callbacks. Don't use them !!
		bool tag_open(std::string tag, std::string version);
		bool tag_close(std::string tag, std::string cdata = "");
		
	private:

        static void start_handler(void* udata, const XML_Char* name, const XML_Char** atts);
        static void character_handler(void* udata, const XML_Char* s, int len);
        static void end_handler(void* udata, const XML_Char* name);

		inline void addTabs();
		
		//streams, d_fout and d_fin are used when it comes to file
		//handling, d_in and d_out are used when actually reading or
		//writing data(either point to d_fout or d_fin or have a
		//separate stream)
		std::ofstream* d_fout;	
		std::ifstream* d_fin;
		std::ostream* d_out;
		std::istream* d_in;

		std::ostringstream* d_outbuf;
		std::istringstream* d_inbuf;

		std::list<std::string> d_tags;

		std::map<std::string, XML_Slot> d_callbacks;
		std::map<std::string, std::string> d_data;
        
		std::string d_last_opened;
        std::string d_version;

		bool d_failed;

		XML_Parser d_parser;
};


#endif //XML_HELPER_H
